#ifndef AMMO_H_
#define AMMO_H_

#include "GRRLIB.h"
#include <list>
#include <vector>
#include "../tools/Color.h"
#include "../math/Polygon.h"
#include "../math/Timer.h"

namespace shootmii {

class Bonus;
class PolyDeg2;
class Animation;
class Terrain;
class Player;
class Manager;
class Rectangle;
class Wind;

const float RECOIL_COEF(.25);

const float SMOKE_WEIGHT(1);
const float WIND_INFLUENCE_ON_SMOKE(.5);
const float SMOKE_AIR_RESISTANCE(.3);

const int MINIMUM_LENGTH_FOR_DAMAGE(16 * 3);
const float DAMAGE_COEF(.5);
const int HIT_DAMAGE_BONUS(20);

const float GRAVITY(-9.81);
const float TIME_STEP(.1);
const int AMMO_WEIGHT(1);
const int AMMO_WIDTH(16);
const int AMMO_HEIGHT(16);
const int AMMO_OVERTAKE(5); // dpassement de la munition du canon

const int GHOST_WIDTH(60);
const int GHOST_HEIGHT(52);
const int GHOST_OFFSET_X(5);
const int GHOST_MARGIN(GHOST_WIDTH - GHOST_HEIGHT / 2);
const int GHOST_DISTANCE(100); // Distance sur laquelle le fondu en transparence s'opre

enum explosionType {
	HIT_THE_GROUND, HIT_ANOTHER_AMMO, HIT_A_PLAYER, HIT_A_SHIELD
};

enum AmmoChild {
	CHILD_GHOST_BUBBLE
};

class Ammo: public Polygon, public Timer {
protected:
	PolyDeg2* calcX;
	PolyDeg2* calcY;
	bool destroyed;
	bool outOfCannon; //< pour que le gestionnaire de collisions ignore l'auto-contact
	bool outOfShield; //< pour que le gestionnaire de collisions ignore l'auto-contact
	bool fired; //< pour savoir si on incrmente le compteur de temps ou non
	bool explosionFinished;
	Terrain* terrain;
	Player* owner;
	Manager* manager;
	bool toDelete;
public:
	Ammo(const float angle, GRRLIB_texImg* image, PolyDeg2* calcX,
			PolyDeg2* calcY, Player* owner, Terrain* terrain, Manager* manager);
	virtual ~Ammo();
	PolyDeg2* getCalcX();
	PolyDeg2* getCalcY();
	int getCol() const;
	int getRow() const;
	Rectangle* getGhostAmmo();
	Rectangle* getGhostAmmo() const;
	Rectangle* getGhostBubble();
	Rectangle* getGhostBubble() const;
	Player* getOwner();
	Player* getOwner() const;
	void computeOut();
	void computeGhost();
	virtual void computeSmoklets();
	virtual void computePosition()=0;
	void compute();
	void setAngle(const float _angle);
	void destroy();
	virtual Animation* destruction(explosionType _type, Player* _playerHit =
			NULL)=0;
	void outCannon();
	void outShield();
	/*
	 * Cette mthode set toDelete  true pour que le gestionnaire de collision la supprime
	 * aprs seulement avoir ger toutes les collisions entre ammos, sinon crash !
	 */
	virtual void deleteMe();
	bool isToDelete();
	bool isOutOfCannon() const;
	bool isOutOfShield() const;
	bool isOffScreen() const;
	bool isTooLow() const;
	bool isDestroyed() const;
	bool hitTheGround(Terrain* terrain) const;
	bool hitABonus(std::list<Bonus*>* bonusList) const;
	Ammo* hitAnotherAmmo(std::list<Ammo*>* ammoList) const;
	Player* hitAPlayer(Player* player1, Player* player2) const;
	bool hitAShield(Player* player);
	void addShieldEffect(Player* player, float angle) const;

	virtual void init(float strength);

	void fire();

};

}

#endif /*AMMO_H_*/
